@@ -1,6 +1,6 @@
 /* Krb5 Auth Applet -- Acquire and release kerberos tickets
  *
- * (C) 2008,2009 Guido Guenther <agx@sigxcpu.org>
+ * (C) 2008,2009,2010 Guido Guenther <agx@sigxcpu.org>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -27,6 +27,7 @@
 #include "krb5-auth-gconf-tools.h"
 #include "krb5-auth-gconf.h"
 #include "krb5-auth-tools.h"
+#include "krb5-auth-tickets.h"
 #ifdef HAVE_LIBNOTIFY
 #include <libnotify/notify.h>
 #endif
@@ -60,6 +61,8 @@ struct _KaApplet {
 
 struct _KaAppletClass {
   GObjectClass parent;
+
+  guint signals [KA_SIGNAL_COUNT];
 };
 
 G_DEFINE_TYPE(KaApplet, ka_applet, G_TYPE_OBJECT);
@@ -72,13 +75,13 @@ struct _KaAppletPrivate
 	const char* icons[3]; 		/* for invalid, expiring and valid tickts */
 	gboolean show_trayicon;		/* show the trayicon */
 
-	KaPwDialog* pwdialog;		/* the password dialog */
+	KaPwDialog *pwdialog;		/* the password dialog */
 	int	   pw_prompt_secs;	/* when to start prompting for a password */
 
 #ifdef HAVE_LIBNOTIFY
 	NotifyNotification* notification;/* notification messages */
-	const char* notify_gconf_key;	/* disable notification gconf key */
 #endif /* HAVE_LIBNOTIFY */
+	const char* notify_gconf_key;	/* disable notification gconf key */
 	char* principal;		/* the principal to request */
 	gboolean renewable;		/* credentials renewable? */
 	char* pk_userid;		/* "userid" for pkint */
@@ -250,6 +253,11 @@ ka_applet_class_init(KaAppletClass *klass)
 {
 	GObjectClass *object_class = G_OBJECT_CLASS(klass);
 	GParamSpec *pspec;
+	const gchar *signalNames [ KA_SIGNAL_COUNT ] = {
+				"krb-tgt-acquired",
+				"krb-tgt-renewed",
+				"krb-tgt-expired"};
+	int i;
 
 	object_class->dispose = ka_applet_dispose;
 	object_class->finalize = ka_applet_finalize;
@@ -329,6 +337,23 @@ ka_applet_class_init(KaAppletClass *klass)
 	g_object_class_install_property (object_class,
                                          KA_PROP_TGT_RENEWABLE,
                                          pspec);
+
+	for (i = 0; i < KA_SIGNAL_COUNT ; i++) {
+		guint signalId;
+		signalId =
+			g_signal_new ( signalNames [i],
+			G_OBJECT_CLASS_TYPE ( klass ),
+			G_SIGNAL_RUN_LAST,
+			0,
+			NULL,
+			NULL,
+			g_cclosure_marshal_VOID__STRING,
+			G_TYPE_NONE,
+			2, /* number of parameters */
+			G_TYPE_STRING,
+			G_TYPE_UINT);
+		klass->signals [i] = signalId ;
+	}
 }
 
 
@@ -389,7 +414,16 @@ ka_show_notification (KaApplet *applet)
 {
 	/* wait for the panel to be settled before showing a bubble */
 	if (gtk_status_icon_is_embedded (applet->priv->tray_icon)) {
-		notify_notification_show (applet->priv->notification, NULL);
+		GError *error = NULL;
+		gboolean ret;
+
+		ret = notify_notification_show (applet->priv->notification, &error);
+		if (!ret) {
+			g_assert (error);
+			g_assert (error->message);
+			g_warning ("Failed to show notification: %s", error->message);
+			g_clear_error (&error);
+		}
 	} else {
 		g_timeout_add_seconds (5, (GSourceFunc)ka_show_notification, applet);
 	}
@@ -464,7 +498,10 @@ ka_send_event_notification (KaApplet *applet G_GNUC_UNUSED,
 #endif /* ! HAVE_LIBNOTIFY */
 
 
-/* update the tray icon's tooltip and icon */
+/*
+ * update the tray icon's tooltip and icon
+ * and notify listeners about acquired/expiring tickets via signals
+ */
 int
 ka_applet_update_status(KaApplet* applet, krb5_timestamp expiry)
 {
@@ -489,6 +526,7 @@ ka_applet_update_status(KaApplet* applet, krb5_timestamp expiry)
 						"krb-valid-ticket",
 						"dont-show-again");
 			}
+			ka_applet_signal_emit (applet, KA_SIGNAL_ACQUIRED_TGT, expiry);
 			expiry_notified = FALSE;
 		} else if (remaining < applet->priv->pw_prompt_secs && (now - last_warn) > NOTIFY_SECONDS &&
 			   !applet->priv->renewable) {
@@ -518,13 +556,14 @@ ka_applet_update_status(KaApplet* applet, krb5_timestamp expiry)
 						"krb-no-valid-ticket",
 						"dont-show-again");
 			}
+			ka_applet_signal_emit (applet, KA_SIGNAL_EXPIRED_TGT, expiry);
 			expiry_notified = TRUE;
 			last_warn = 0;
 		}
 	}
 
 	gtk_status_icon_set_from_icon_name (applet->priv->tray_icon, tray_icon);
-	gtk_status_icon_set_tooltip (applet->priv->tray_icon, tooltip_text);
+	gtk_status_icon_set_tooltip_text (applet->priv->tray_icon, tooltip_text);
 	g_free(tooltip_text);
 	return 0;
 }
@@ -544,9 +583,29 @@ static void
 ka_applet_cb_preferences (GtkWidget* menuitem G_GNUC_UNUSED,
                           gpointer user_data G_GNUC_UNUSED)
 {
+	GError *error = NULL;
+
 	g_spawn_command_line_async (BIN_DIR
 				    G_DIR_SEPARATOR_S
-				    "krb5-auth-dialog-preferences", NULL);
+				    "krb5-auth-dialog-preferences",
+				    &error);
+	if (error) {
+		GtkWidget *message_dialog;
+
+		message_dialog = gtk_message_dialog_new (NULL,
+					GTK_DIALOG_DESTROY_WITH_PARENT,
+					GTK_MESSAGE_ERROR,
+					GTK_BUTTONS_CLOSE,
+					_("There was an error launching the preferences dialog: %s"),
+					error->message);
+		gtk_window_set_resizable (GTK_WINDOW (message_dialog), FALSE);
+
+		g_signal_connect (message_dialog, "response",
+				  G_CALLBACK (gtk_widget_destroy),
+				  NULL);
+		gtk_widget_show (message_dialog);
+		g_error_free (error);
+	  }
 }
 
 
@@ -599,7 +658,7 @@ ka_applet_cb_about_dialog (GtkMenuItem* menuitem G_GNUC_UNUSED,
 				"Christopher Aillon <caillon@redhat.com>",
 				"Jonathan Blandford <jrb@redhat.com>",
 				"Colin Walters <walters@verbum.org>",
-				"Guido Günther <agx@sigxpcu.org>",
+				"Guido Günther <agx@sigxcpu.org>",
 				NULL };
 
 	gtk_about_dialog_set_url_hook (ka_about_dialog_url_hook, NULL, NULL);
@@ -637,6 +696,13 @@ ka_applet_cb_destroy_ccache(GtkMenuItem* menuitem G_GNUC_UNUSED,
 	ka_destroy_ccache(applet);
 }
 
+static void
+ka_applet_cb_show_tickets(GtkMenuItem* menuitem G_GNUC_UNUSED,
+			  gpointer user_data G_GNUC_UNUSED)
+{
+	ka_tickets_dialog_run();
+}
+
 
 /* The tray icon's context menu */
 static gboolean
@@ -658,6 +724,12 @@ ka_applet_create_context_menu (KaApplet* applet)
 
 	ka_applet_menu_add_separator_item (menu);
 
+	/* Ticket dialog */
+	menu_item = gtk_image_menu_item_new_with_mnemonic("_List Tickets");
+	g_signal_connect (G_OBJECT (menu_item), "activate",
+			  G_CALLBACK (ka_applet_cb_show_tickets), applet);
+	gtk_menu_shell_append (GTK_MENU_SHELL (menu), menu_item);
+
 	/* Preferences */
 	menu_item = gtk_image_menu_item_new_from_stock(GTK_STOCK_PREFERENCES, NULL);
 	g_signal_connect (G_OBJECT (menu_item), "activate",
@@ -744,7 +816,8 @@ ka_applet_create_tray_icon (KaApplet* applet)
 			  "popup-menu",
 			  G_CALLBACK(ka_tray_icon_on_menu), applet);
 	gtk_status_icon_set_from_icon_name (tray_icon, applet->priv->icons[exp_icon]);
-	gtk_status_icon_set_tooltip (tray_icon, PACKAGE);
+	gtk_status_icon_set_tooltip_text (tray_icon, PACKAGE);
+	gtk_status_icon_set_title (tray_icon, KA_NAME);
         applet->priv->tray_icon = tray_icon;
 	return TRUE;
 }
@@ -791,11 +864,28 @@ ka_applet_get_pwdialog(const KaApplet* applet)
 	return applet->priv->pwdialog;
 }
 
+void
+ka_applet_signal_emit (KaApplet* this, KaAppletSignalNumber signum,
+		       krb5_timestamp expiry)
+{
+	KaAppletClass *klass = KA_APPLET_GET_CLASS (this);
+	char *princ;
+
+	princ = ka_unparse_name ();
+	if (!princ)
+		return;
+
+	g_signal_emit (this, klass->signals[signum], 0, princ, (guint32)expiry);
+	g_free (princ);
+}
+
 /* create the tray icon applet */
 KaApplet*
 ka_applet_create()
 {
 	KaApplet* applet = ka_applet_new();
+	GError *error = NULL;
+	gboolean ret;
 
 	if (!(ka_applet_setup_icons (applet)))
 		g_error ("Failure to setup icons");
@@ -808,15 +898,22 @@ ka_applet_create()
 	                  G_CALLBACK (ka_applet_cb_show_trayicon), NULL);
 
 	applet->priv->uixml = gtk_builder_new();
-	g_assert(gtk_builder_add_from_file(applet->priv->uixml,
-					   KA_DATA_DIR G_DIR_SEPARATOR_S
-				           PACKAGE ".xml", NULL));
+	ret = gtk_builder_add_from_file(applet->priv->uixml,
+					KA_DATA_DIR G_DIR_SEPARATOR_S
+				        PACKAGE ".xml", &error);
+	if (!ret) {
+		g_assert (error);
+		g_assert (error->message);
+		g_error ("Failed to load UI XML: %s", error->message);
+	}
 	applet->priv->pwdialog = ka_pwdialog_create(applet->priv->uixml);
 	g_return_val_if_fail (applet->priv->pwdialog != NULL, NULL);
 
 	applet->priv->gconf = ka_gconf_init (applet);
 	g_return_val_if_fail (applet->priv->gconf != NULL, NULL);
 
+	ka_tickets_dialog_create(applet->priv->uixml);
+
 	return applet;
 }
 
